# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async process mapset test case admin
"""
import unittest
from flask.json import loads as json_loads, dumps as json_dumps
from flask.json import loads as json_load
import time

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

# Module change example for r.slope.aspect with g.region adjustment
process_chain_long = {
    1: {
        "module": "g.region",
        "inputs": {"raster": "elevation@PERMANENT"},
        "flags": "p",
        "verbose": True,
    },
    2: {
        "module": "r.slope.aspect",
        "inputs": {
            "elevation": "elevation@PERMANENT",
            "format": "degrees",
            "min_slope": "0.0",
        },
        "outputs": {
            "aspect": {"name": "my_aspect"},
            "slope": {"name": "my_slope"},
        },
        "flags": "a",
        "overwrite": False,
        "verbose": True,
    },
    3: {
        "module": "r.watershed",
        "inputs": {"elevation": "elevation@PERMANENT"},
        "outputs": {"accumulation": {"name": "my_accumulation"}},
    },
    4: {
        "module": "r.info",
        "inputs": {"map": "my_aspect"},
        "flags": "gr",
        "verbose": True,
    },
}

process_chain_short = {
    1: {
        "module": "g.region",
        "inputs": {"raster": "elevation@PERMANENT"},
        "flags": "p",
        "verbose": True,
    },
    2: {
        "module": "r.slope.aspect",
        "inputs": {
            "elevation": "elevation@PERMANENT",
            "format": "degrees",
            "min_slope": "0.0",
        },
        "outputs": {
            "aspect": {"name": "my_aspect_2"},
            "slope": {"name": "my_slope_2"},
        },
        "flags": "a",
        "overwrite": False,
        "verbose": True,
    },
}


class AsyncProcessMapsetTestCaseAdmin(ActiniaResourceTestCaseBase):
    def check_remove_test_mapset(self):
        rv = self.server.get(
            URL_PREFIX + "/locations/nc_spm_08/mapsets",
            headers=self.user_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        mapsets = json_load(rv.data)["process_results"]

        if "test_mapset" in mapsets:
            # Delete the mapset if it already exists
            rv = self.server.delete(
                URL_PREFIX + "/locations/nc_spm_08/mapsets/test_mapset",
                headers=self.admin_auth_header,
            )
            print(rv.data)
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )
            self.assertEqual(
                rv.mimetype,
                "application/json",
                "Wrong mimetype %s" % rv.mimetype,
            )

    def test_1_new_mapset(self):
        """
        Create a new mapset and process
        :return:
        """
        self.check_remove_test_mapset()

        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/test_mapset/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)

        rv = self.server.get(
            URL_PREFIX + "/locations/nc_spm_08/mapsets",
            headers=self.admin_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        mapsets = json_load(rv.data)["process_results"]

        self.assertTrue("test_mapset" in mapsets)

        rv = self.server.get(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/test_mapset/raster_layers",
            headers=self.admin_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        map_list = json_load(rv.data)["process_results"]
        self.assertTrue("my_accumulation" in map_list)
        self.assertTrue("my_aspect" in map_list)
        self.assertTrue("my_slope" in map_list)

        # Remove the mapset
        rv = self.server.delete(
            URL_PREFIX + "/locations/nc_spm_08/mapsets/test_mapset",
            headers=self.admin_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_2_existing_mapset(self):
        """
        An existing mapset and process
        :return:
        """
        self.check_remove_test_mapset()

        # Create new mapset
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/mapsets/test_mapset",
            headers=self.admin_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Run the processing using an existing mapset
        # Atemporary mapset will be created and merged in the existing
        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/test_mapset/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)

        rv = self.server.get(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/test_mapset/raster_layers",
            headers=self.user_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        map_list = json_load(rv.data)["process_results"]
        self.assertTrue("my_accumulation" in map_list)
        self.assertTrue("my_aspect" in map_list)
        self.assertTrue("my_slope" in map_list)

        # Remove the mapset
        rv = self.server.delete(
            URL_PREFIX + "/locations/nc_spm_08/mapsets/test_mapset",
            headers=self.admin_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_3_existing_mapset_lock(self):
        """
        Try to run two request on a locked mapset
        :return:
        """

        self.check_remove_test_mapset()

        # Create new mapset
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/mapsets/test_mapset",
            headers=self.admin_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Run the processing inside the new mapset
        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/test_mapset/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )

        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        resp = json_loads(rv.data)

        rv_user_id = resp["user_id"]
        rv_resource_id = resp["resource_id"]

        time.sleep(0.5)

        # Try to lock again and again
        # Run the processing inside the new mapset
        # Second runner
        rv_lock_1 = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/test_mapset/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_short),
            content_type="application/json",
        )

        print(rv_lock_1.data)
        self.assertEqual(
            rv_lock_1.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv_lock_1.mimetype,
            "application/json",
            "Wrong mimetype %s" % rv.mimetype,
        )
        # Third runner
        rv_lock_2 = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/test_mapset/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )

        print(rv_lock_2.data)
        self.assertEqual(
            rv_lock_2.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv_lock_2.mimetype,
            "application/json",
            "Wrong mimetype %s" % rv.mimetype,
        )

        # Check the first runner
        while True:
            rv = self.server.get(
                URL_PREFIX + "/resources/%s/%s" % (rv_user_id, rv_resource_id),
                headers=self.admin_auth_header,
            )
            print(rv.data)
            resp = json_loads(rv.data)
            if resp["status"] == "finished" or resp["status"] == "error":
                break
            time.sleep(0.2)

        self.assertEqual(resp["status"], "finished")
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )

        # Check the second runner
        resp = json_loads(rv_lock_1.data)
        rv_user_id = resp["user_id"]
        rv_resource_id = resp["resource_id"]
        while True:
            rv = self.server.get(
                URL_PREFIX + "/resources/%s/%s" % (rv_user_id, rv_resource_id),
                headers=self.admin_auth_header,
            )
            print(rv.data)
            resp = json_loads(rv.data)
            if resp["status"] == "finished" or resp["status"] == "error":
                break
            time.sleep(0.2)

        self.assertEqual(resp["status"], "error")
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )

        # Check the third runner
        resp = json_loads(rv_lock_2.data)
        rv_user_id = resp["user_id"]
        rv_resource_id = resp["resource_id"]
        # Check the first runner
        while True:
            rv = self.server.get(
                URL_PREFIX + "/resources/%s/%s" % (rv_user_id, rv_resource_id),
                headers=self.admin_auth_header,
            )
            print(rv.data)
            resp = json_loads(rv.data)
            if resp["status"] == "finished" or resp["status"] == "error":
                break
            time.sleep(0.2)

        self.assertEqual(resp["status"], "error")
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )

        # Remove the mapset
        rv = self.server.delete(
            URL_PREFIX + "/locations/nc_spm_08/mapsets/test_mapset",
            headers=self.admin_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_4_create_global_mapset(self):
        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/PERMANENT/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )


if __name__ == "__main__":
    unittest.main()



#!flask/bin/python
# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Actinia core Endpoint definitions
"""
import traceback
import sys
from pprint import pprint
from actinia_core.core.common.app import flask_api
from actinia_core.core.common.config import global_config
from actinia_core.core.logging_interface import log
from actinia_core.rest.location_management import (
    ListLocationsResource,
    LocationManagementResourceUser,
)
from actinia_core.rest.location_management import (
    LocationManagementResourceAdminUser,
)
from actinia_core.rest.mapsets import AllMapsetsListingResourceAdmin
from actinia_core.rest.mapset_management import (
    ListMapsetsResource,
    MapsetManagementResourceUser,
)
from actinia_core.rest.mapset_management import (
    MapsetLockManagementResource,
    MapsetManagementResourceAdmin,
)
from actinia_core.rest.strds_management import (
    STRDSManagementResource,
    SyncSTRDSListerResource,
)
from actinia_core.rest.strds_raster_management import STRDSRasterManagement
from actinia_core.rest.raster_layer import RasterLayerResource
from actinia_core.rest.map_layer_management import RasterLayersResource
from actinia_core.rest.map_layer_management import VectorLayersResource
from actinia_core.rest.vector_layer import VectorLayerResource
from actinia_core.rest.ephemeral_processing import AsyncEphemeralResource
from actinia_core.rest.ephemeral_processing_with_export import (
    AsyncEphemeralExportResource,
)
from actinia_core.rest.ephemeral_processing_with_export import (
    AsyncEphemeralExportS3Resource,
)
from actinia_core.rest.ephemeral_processing_with_export import (
    AsyncEphemeralExportGCSResource,
)
from actinia_core.rest.persistent_mapset_merger import (
    AsyncPersistentMapsetMergerResource,
)
from actinia_core.rest.raster_export import (
    AsyncEphemeralRasterLayerRegionExporterResource,
)
from actinia_core.rest.raster_export import (
    AsyncEphemeralRasterLayerExporterResource,
)
from actinia_core.rest.persistent_processing import AsyncPersistentResource
from actinia_core.rest.ephemeral_custom_processing import (
    AsyncEphemeralCustomResource,
)
from actinia_core.rest.process_validation import AsyncProcessValidationResource
from actinia_core.rest.process_validation import SyncProcessValidationResource
from actinia_core.rest.user_management import (
    UserListResource,
    UserManagementResource,
)
from actinia_core.rest.api_log_management import APILogResource
from actinia_core.rest.user_api_key import (
    TokenCreationResource,
    APIKeyCreationResource,
)
from actinia_core.rest.resource_management import (
    ResourceManager,
    ResourcesManager,
    ResourceIterationManager,
)
from actinia_core.rest.resource_streamer import RequestStreamerResource
from actinia_core.rest.download_cache_management import (
    SyncDownloadCacheResource,
)
from actinia_core.rest.resource_storage_management import (
    SyncResourceStorageResource,
)
from actinia_core.rest.vector_renderer import (
    SyncEphemeralVectorRendererResource,
)
from actinia_core.rest.raster_legend import SyncEphemeralRasterLegendResource
from actinia_core.rest.raster_colors import SyncPersistentRasterColorsResource
from actinia_core.rest.raster_renderer import (
    SyncEphemeralRasterRendererResource,
)
from actinia_core.rest.raster_renderer import (
    SyncEphemeralRasterRGBRendererResource,
)
from actinia_core.rest.raster_renderer import (
    SyncEphemeralRasterShapeRendererResource,
)
from actinia_core.rest.strds_renderer import SyncEphemeralSTRDSRendererResource
from actinia_core.rest.process_chain_monitoring import (
    MaxMapsetSizeResource,
    MapsetSizeResource,
    MapsetSizeRenderResource,
    MapsetSizeDiffResource,
    MapsetSizeDiffRenderResource,
)


__license__ = "GPLv3"
__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


def create_core_endpoints():
    # Locationmanagement
    flask_api.add_resource(ListLocationsResource, "/locations")
    flask_api.add_resource(
        LocationManagementResourceUser,
        "/locations/<string:location_name>/info",
    )
    flask_api.add_resource(
        LocationManagementResourceAdminUser,
        "/locations/<string:location_name>",
    )
    # Mapset management
    flask_api.add_resource(
        ListMapsetsResource, "/locations/<string:location_name>/mapsets"
    )
    flask_api.add_resource(
        MapsetManagementResourceUser,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/info",
    )
    flask_api.add_resource(
        MapsetManagementResourceAdmin,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>",
    )
    flask_api.add_resource(
        MapsetLockManagementResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/lock",
    )

    # Raster management
    flask_api.add_resource(
        RasterLayersResource,
        "/locations/<string:location_name>/mapsets/"
        "<string:mapset_name>/raster_layers",
    )
    flask_api.add_resource(
        RasterLayerResource,
        "/locations/<string:location_name>/mapsets/"
        "<string:mapset_name>/raster_layers/<string:raster_name>",
    )
    flask_api.add_resource(
        SyncEphemeralRasterLegendResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/"
        "raster_layers/<string:raster_name>/legend",
    )
    flask_api.add_resource(
        SyncPersistentRasterColorsResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/"
        "raster_layers/<string:raster_name>/colors",
    )
    flask_api.add_resource(
        SyncEphemeralRasterRendererResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/"
        "raster_layers/<string:raster_name>/render",
    )
    flask_api.add_resource(
        SyncEphemeralRasterRGBRendererResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/"
        "render_rgb",
    )
    flask_api.add_resource(
        SyncEphemeralRasterShapeRendererResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/"
        "render_shade",
    )
    # STRDS management
    flask_api.add_resource(
        SyncSTRDSListerResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/strds",
    )
    flask_api.add_resource(
        STRDSManagementResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/strds/"
        "<string:strds_name>",
    )
    flask_api.add_resource(
        STRDSRasterManagement,
        "/locations/<string:location_name>/mapsets/"
        "<string:mapset_name>/strds/<string:strds_name>/raster_layers",
    )
    # Vector management
    flask_api.add_resource(
        VectorLayersResource,
        "/locations/<string:location_name>/mapsets/"
        "<string:mapset_name>/vector_layers",
    )
    flask_api.add_resource(
        VectorLayerResource,
        "/locations/<string:location_name>/mapsets/"
        "<string:mapset_name>/vector_layers/<string:vector_name>",
    )
    flask_api.add_resource(
        SyncEphemeralVectorRendererResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/"
        "vector_layers/<string:vector_name>/render",
    )
    flask_api.add_resource(
        SyncEphemeralSTRDSRendererResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/"
        "strds/<string:strds_name>/render",
    )

    # Validation
    flask_api.add_resource(
        AsyncProcessValidationResource,
        "/locations/<string:location_name>/process_chain_validation_async",
    )

    flask_api.add_resource(
        SyncProcessValidationResource,
        "/locations/<string:location_name>/process_chain_validation_sync",
    )
    # Async processing
    flask_api.add_resource(
        AsyncEphemeralCustomResource, "/custom_process/<string:executable>"
    )
    flask_api.add_resource(
        AsyncEphemeralResource,
        "/locations/<string:location_name>/processing_async",
    )
    flask_api.add_resource(
        AsyncEphemeralExportResource,
        "/locations/<string:location_name>/processing_async_export",
    )
    flask_api.add_resource(
        AsyncEphemeralExportS3Resource,
        "/locations/<string:location_name>/processing_async_export_s3",
    )
    flask_api.add_resource(
        AsyncEphemeralExportGCSResource,
        "/locations/<string:location_name>/processing_async_export_gcs",
    )
    flask_api.add_resource(
        AsyncPersistentResource,
        "/locations/<string:location_name>/mapsets/"
        "<string:mapset_name>/processing_async",
    )
    flask_api.add_resource(
        AsyncPersistentMapsetMergerResource,
        "/locations/<string:location_name>/mapsets/"
        "<string:mapset_name>/merging_async",
    )
    flask_api.add_resource(
        AsyncEphemeralRasterLayerExporterResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>/"
        "raster_layers/<string:raster_name>/geotiff_async",
    )
    flask_api.add_resource(
        AsyncEphemeralRasterLayerRegionExporterResource,
        "/locations/<string:location_name>/mapsets/<string:mapset_name>"
        "/raster_layers/<string:raster_name>/geotiff_async_orig",
    )

    # all mapsets across all locations listing
    flask_api.add_resource(AllMapsetsListingResourceAdmin, "/mapsets")

    # User management
    flask_api.add_resource(UserListResource, "/users")
    flask_api.add_resource(UserManagementResource, "/users/<string:user_id>")
    flask_api.add_resource(
        TokenCreationResource,
        "/token",
    )
    flask_api.add_resource(
        APIKeyCreationResource,
        "/api_key",
    )
    flask_api.add_resource(APILogResource, "/api_log/<string:user_id>")

    # Resource management
    """
    The endpoint '/resources/<string:user_id>/<string:resource_id>' has two
    different answers depending on the resource_id. If the resoucre_id starts
    with 'resoucre-id' the latest iteration of the resoucre is given back.
    If the resocue_id is only the id then all iterations of the resource are
    given in the response.
    """
    flask_api.add_resource(
        ResourceManager, "/resources/<string:user_id>/<string:resource_id>"
    )
    flask_api.add_resource(ResourcesManager, "/resources/<string:user_id>")
    flask_api.add_resource(
        ResourceIterationManager,
        "/resources/<string:user_id>/<string:resource_id>/<int:iteration>",
    )
    flask_api.add_resource(
        RequestStreamerResource,
        "/resources/<string:user_id>/<string:resource_id>/<string:file_name>",
    )

    # Download and resource management
    flask_api.add_resource(SyncDownloadCacheResource, "/download_cache")
    flask_api.add_resource(SyncResourceStorageResource, "/resource_storage")

    # Endpoints for monitoring a process chain
    flask_api.add_resource(
        MapsetSizeResource,
        "/resources/<string:user_id>/<string:resource_id>/mapsetsizes",
    )
    flask_api.add_resource(
        MaxMapsetSizeResource,
        "/resources/<string:user_id>/<string:resource_id>/mapsetsizes/max",
    )
    flask_api.add_resource(
        MapsetSizeRenderResource,
        "/resources/<string:user_id>/<string:resource_id>/mapsetsizes/render",
    )
    flask_api.add_resource(
        MapsetSizeDiffResource,
        "/resources/<string:user_id>/<string:resource_id>/mapsetsizes/diffs",
    )
    flask_api.add_resource(
        MapsetSizeDiffRenderResource,
        "/resources/<string:user_id>/<string:resource_id>/mapsetsizes/diffs/"
        "render",
    )


def check_import_plugins():
    import_str = (
        "from {}.endpoints import create_endpoints as create_plugin_endpoints"
        "\ncreate_plugin_endpoints(flask_api=flask_api)\n"
    )
    for plugin in global_config.PLUGINS:
        import_run_str = import_str.format(plugin)
        log.info("Loading plugin %s", plugin)
        exec(import_run_str)


def create_endpoints():
    create_core_endpoints()
    try:
        check_import_plugins()
    except Exception:
        e_type, e_value, e_tb = sys.exc_info()
        pprint(
            dict(
                message=str(e_value),
                traceback=str(traceback.format_tb(e_tb)),
                type=str(e_type),
            )
        )



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Resource storage test case
"""
from flask.json import loads as json_load
from datetime import datetime, timedelta
import unittest
import os

try:
    from .test_resource_base import (
        ActiniaResourceTestCaseBase,
        global_config,
        URL_PREFIX,
    )
except ModuleNotFoundError:
    from test_resource_base import (
        ActiniaResourceTestCaseBase,
        global_config,
        URL_PREFIX,
    )

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class ResourceStorageTestCase(ActiniaResourceTestCaseBase):
    def test_resource_storage(self):
        # global_config.GRASS_RESOURCE_DIR = "/tmp/rstorage_tmp"
        global_config.GRASS_RESOURCE_QUOTA = 1
        try:
            os.mkdir(global_config.GRASS_RESOURCE_DIR)
        except Exception:  # more precise exception gladly accepted
            pass

        try:
            os.mkdir(
                os.path.join(global_config.GRASS_RESOURCE_DIR, self.admin_id)
            )
        except Exception:  # more precise exception gladly accepted
            pass

        rv = self.server.get(
            URL_PREFIX + "/resource_storage", headers=self.admin_auth_header
        )
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        self.assertTrue("used" in json_load(rv.data)["process_results"])
        self.assertTrue("quota" in json_load(rv.data)["process_results"])
        self.assertTrue("free" in json_load(rv.data)["process_results"])
        self.assertTrue(
            "free_percent" in json_load(rv.data)["process_results"]
        )

        rv = self.server.delete(
            URL_PREFIX + "/resource_storage", headers=self.admin_auth_header
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        rv = self.server.get(
            URL_PREFIX + "/resource_storage", headers=self.admin_auth_header
        )
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        self.assertTrue("used" in json_load(rv.data)["process_results"])
        self.assertTrue("quota" in json_load(rv.data)["process_results"])
        self.assertTrue("free" in json_load(rv.data)["process_results"])
        self.assertTrue(
            "free_percent" in json_load(rv.data)["process_results"]
        )

    # TODO: configure wrong resource dir path differently (via config file)
    # see https://github.com/actinia-org/actinia-core/issues/422
    # def test_resource_storage_error_1(self):
    #     global_config.GRASS_RESOURCE_DIR = "/tmp/rstorage_tmp_nope"
    #     global_config.GRASS_RESOURCE_QUOTA = 1

    #     rv = self.server.get(
    #         URL_PREFIX + "/resource_storage", headers=self.admin_auth_header
    #     )
    #     import pdb; pdb.set_trace()
    #     self.assertEqual(
    #         rv.status_code,
    #         400,
    #         "HTML status code is wrong %i" % rv.status_code,
    #     )
    #     self.assertEqual(
    #         rv.mimetype,
    #         "application/json",
    #         "Wrong mimetype %s" % rv.mimetype
    #     )

    # TODO: configure wrong resource dir path differently (via config file)
    # see https://github.com/actinia-org/actinia-core/issues/422
    # def test_resource_storage_error_2(self):
    #     global_config.GRASS_RESOURCE_DIR = "/tmp/rstorage_tmp_nope"
    #     global_config.GRASS_RESOURCE_QUOTA = 1

    #     rv = self.server.delete(
    #         URL_PREFIX + "/resource_storage", headers=self.admin_auth_header
    #     )
    #     self.waitAsyncStatusAssertHTTP(
    #         rv,
    #         headers=self.admin_auth_header,
    #         http_status=400,
    #         status="error",
    #         message_check="AsyncProcessError:",
    #     )
    #     self.assertEqual(
    #         rv.mimetype,
    #         "application/json",
    #         "Wrong mimetype %s" % rv.mimetype
    #     )

    def test_resource_storage_delete_olderthan(self):
        # global_config.GRASS_RESOURCE_DIR = "/tmp/rstorage_tmp"
        global_config.GRASS_RESOURCE_QUOTA = 1
        try:
            os.mkdir(global_config.GRASS_RESOURCE_DIR)
        except Exception:  # more precise exception gladly accepted
            pass

        admin_resource_path = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            self.admin_id,
        )
        try:
            os.mkdir(admin_resource_path)
        except Exception:  # more precise exception gladly accepted
            pass

        # create files from specified date
        now = datetime.now()
        before_15days = now - timedelta(days=15)
        file1 = os.path.join(admin_resource_path, "file1.txt")
        file2 = os.path.join(admin_resource_path, "file2.txt")
        date1 = now.strftime("%Y-%m-%d %H:%M:%S")
        date2 = before_15days.strftime("%Y-%m-%d %H:%M:%S")
        cmd_touch_file1 = f"touch -d '{date1}' {file1}"
        cmd_touch_file2 = f"touch -d '{date2}' {file2}"
        os.system(cmd_touch_file1)
        os.system(cmd_touch_file2)
        created_files = os.listdir(admin_resource_path)
        self.assertIn(
            "file1.txt",
            created_files,
            "'file1.txt' not in resource path after creation",
        )
        self.assertIn(
            "file2.txt",
            created_files,
            "'file2.txt' not in resource path after creation",
        )

        # request resource storage
        rv = self.server.get(
            URL_PREFIX + "/resource_storage", headers=self.admin_auth_header
        )
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )

        # delete files older than 10 days
        rv = self.server.delete(
            URL_PREFIX + "/resource_storage?olderthan=10",
            headers=self.admin_auth_header,
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)
        self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        # check files
        files = os.listdir(admin_resource_path)
        self.assertIn("file1.txt", files, "'file1.txt' not in resource path")
        self.assertNotIn("file2.txt", files, "'file2.txt' in resource path")

        # clean up resource storage
        rv = self.server.delete(
            URL_PREFIX + "/resource_storage", headers=self.admin_auth_header
        )
        self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Common test case base
"""
import atexit
import unittest
import signal
import time
import os
from actinia_core.core.common.config import global_config
import actinia_core.core.common.redis_interface as redis_interface
from actinia_core.core.common.app import flask_app

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2019, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

redis_pid = None
custom_actinia_cfg = False

# Set this variable to use a actinia config file in a docker container
if "ACTINIA_CUSTOM_TEST_CFG" in os.environ:
    custom_actinia_cfg = str(os.environ["ACTINIA_CUSTOM_TEST_CFG"])


def setup_environment():
    # If docker config
    if custom_actinia_cfg is not False:
        global_config.read(custom_actinia_cfg)
        return

    global redis_pid
    # Set the port to the test redis server
    global_config.REDIS_SERVER_SERVER = "localhost"
    global_config.REDIS_SERVER_PORT = 7000

    # home = os.getenv("HOME")

    # GRASS

    # Setup the test environment
    global_config.GRASS_GIS_BASE = "/usr/local/grass/"
    global_config.GRASS_GIS_START_SCRIPT = "/usr/local/bin/grass"
    # global_config.GRASS_DATABASE= "/usr/local/grass_test_db"
    # global_config.GRASS_DATABASE = "%s/actinia/grass_test_db" % home

    # Start the redis server for user and logging management
    redis_pid = os.spawnl(
        os.P_NOWAIT,
        "/usr/bin/redis-server",
        "common/redis.conf",
        "--port %i" % global_config.REDIS_SERVER_PORT,
    )
    time.sleep(1)


def stop_redis():
    global redis_pid
    # Kill th redis server
    if custom_actinia_cfg is not False:
        return
    os.kill(redis_pid, signal.SIGTERM)


# Register the redis stop function
atexit.register(stop_redis)
# Setup the environment
setup_environment()


class CommonTestCaseBase(unittest.TestCase):
    """
    This is the base class for the common testing
    """

    @classmethod
    def setUpClass(cls):
        if custom_actinia_cfg is not False:
            global_config.read(custom_actinia_cfg)
        else:
            global_config.REDIS_SERVER_URL = "localhost"
            global_config.REDIS_SERVER_PORT = 7000

        args = (
            global_config.REDIS_SERVER_URL,
            global_config.REDIS_SERVER_PORT,
        )
        if (
            global_config.REDIS_SERVER_PW
            and global_config.REDIS_SERVER_PW is not None
        ):
            args = (*args, global_config.REDIS_SERVER_PW)

        redis_interface.connect(*args)

    @classmethod
    def tearDownClass(cls):
        redis_interface.disconnect()

    def setUp(self):
        # We need to set the application context
        self.app_context = flask_app.app_context()
        self.app_context.push()

    def tearDown(self):
        self.app_context.pop()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Redis server interface for API logging
"""

from actinia_core.core.common.redis_base import RedisBaseInterface

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class RedisAPILogInterface(RedisBaseInterface):
    """
    The Redis API log database interface
    """

    # API logging entries are lists in the Redis database using LPUSH, LTRIM,
    # LRANGE for management
    api_log_prefix = "API-LOG-LIST::"

    def __init__(self):
        RedisBaseInterface.__init__(self)

    """
    ########################## API  LOG #######################################
    """

    """
    The API logs are organized by lists that have as key the user id.
    Each API call from a user_id is logged in its dedicated list with
    lpush calls. For each user_id exists a dedicated list of API calls.

    Log entries for a user id can be listed trimmed and the log can be
    completely deleted.

    ..code:

    user_id_1 : log entry 1
                log entry 2
                log entry 3

    user_id_2 : log entry 1
                log entry 2
                log entry 3

    """

    def add(self, user_id, log_entry):
        """Add a API log entry to a user specific  API log list in the Redis
        server

        Args:
            user_id (str): The user id of the API log
            log_entry (str): A string

        Returns:
            int:
            The index of the new entry in the api log list

        """
        return self.redis_server.lpush(
            self.api_log_prefix + user_id, log_entry
        )

    def list(self, user_id, start, end):
        """Return all API log entries between start and end indices

        Args:
            user_id (str): The user id of the API log
            start (int): Integer start index
            end (int): Integer end index

        Returns:
            list:
            A list of user specific API log entries

        """
        return self.redis_server.lrange(
            self.api_log_prefix + user_id, start, end
        )

    def trim(self, user_id, start, end):
        """Remove all API log entries outside start and end indices

        Args:
            user_id (str): The user id of the API log
            start (int): Integer start index
            end (int): Integer end index

        Returns:
            bool:
            True in any case

        """
        return self.redis_server.ltrim(
            self.api_log_prefix + user_id, start, end
        )

    def size(self, user_id):
        """Return the number of entries in the api log list

        Args:
            user_id (str): The user id of the API log

        Returns:
            int:
            The number of entries in the api log list

        """
        return self.redis_server.llen(self.api_log_prefix + user_id)

    def delete(self, user_id):
        """Remove the log list

        Args:
            user_id (str): The user id of the API log

        Returns:
            bool:
            True in case of success, False otherwise

        """
        return bool(self.redis_server.delete(self.api_log_prefix + user_id))


# Create the Redis interface instance
redis_api_log_interface = RedisAPILogInterface()


def test_api_logging(r):
    user_id = "abcdefg"

    # Remove the loglist
    r.delete(user_id)

    ret = r.add(user_id, "API-log entry 1")
    if ret != 1:
        raise Exception("add does not work")

    ret = r.add(user_id, "API-log entry 2")
    if ret != 2:
        raise Exception("add does not work")

    ret = r.add(user_id, "API-log entry 3")
    if ret != 3:
        raise Exception("add does not work")

    print(r.list(user_id, 0, -1))

    ret = r.size(user_id)
    if ret != 3:
        raise Exception("size does not work")

    if r.trim(user_id, 0, 1) is not True:
        raise Exception("trim_log_entries does not work")

    ret = r.size(user_id)
    if ret != 2:
        raise Exception("size does not work")

    # Remove the loglist
    if r.delete(user_id) is not True:
        raise Exception("delete does not work")


if __name__ == "__main__":
    import os
    import signal
    import time

    pid = os.spawnl(
        os.P_NOWAIT, "/usr/bin/redis-server", "./redis.conf", "--port 7000"
    )

    time.sleep(1)

    try:
        r = RedisAPILogInterface()
        r.connect(host="localhost", port=7000)
        test_api_logging(r)
        r.disconnect()
    except Exception:
        raise
    finally:
        os.kill(pid, signal.SIGTERM)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2021-2023 mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Interim Result class
"""

import os
import subprocess
import shutil
from fnmatch import filter
from .messages_logger import MessageLogger
from actinia_core.core.common.config import global_config, DEFAULT_CONFIG_PATH
from actinia_core.core.common.exceptions import RsyncError
from actinia_core.core.mapset_merge_utils import change_mapsetname

__license__ = "GPLv3"
__author__ = "Anika Weinmann, Lina Krisztian"
__copyright__ = "Copyright 2021-2023, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"
__email__ = "info@mundialis.de"


def get_directory_size(directory):
    """Returns the directory size in bytes.
    Args:
        directory (string): The path to a directory

    Returns:
        total: the size of the directory in bytes

    """
    total = 0
    try:
        for entry in os.scandir(directory):
            if entry.is_file():
                total += os.path.getsize(entry)
            elif entry.is_dir():
                total += get_directory_size(entry.path)
    except NotADirectoryError:
        return os.path.getsize(directory)
    except PermissionError:
        return 0
    return total


class InterimResult(object):
    """This class manages the interim results"""

    def __init__(self, user_id, resource_id, iteration, endpoint):
        """Init method for InterimResult class
        Args:
            user_id (str): The unique user name/id
            resource_id (str): The id of the resource
            old_pc_step (int): The number of the successfully finished steps of
                               the process chain in the previous iteration
        """
        global_config.read(DEFAULT_CONFIG_PATH)
        self.logger = MessageLogger()
        self.user_resource_interim_storage_path = os.path.join(
            global_config.GRASS_RESOURCE_DIR, user_id, "interim"
        )
        self.saving_interim_results = global_config.SAVE_INTERIM_RESULTS
        self.resource_id = resource_id
        self.iteration = iteration if iteration is not None else 1
        self.old_pc_step = None
        self.endpoint = endpoint
        self.include_additional_mapset_pattern = (
            global_config.INCLUDE_ADDITIONAL_MAPSET_PATTERN
        )

    def set_old_pc_step(self, old_pc_step):
        """Set method for the number of the successfully finished steps of
        the process chain in the previous iteration

        Args:
            old_pc_step (int): The number of the successfully finished steps of
                               the process chain in the previous iteration
        """
        self.old_pc_step = old_pc_step

    def _get_step_folder_name(self, pc_step):
        """Return the name of the interim folder for the process chain step
        Args:
            pc_step (int): The number of the step in the process chain where to
                           continue

        Returns:
            (str): The name of the interim folder for the process chain step

        """
        return f"step{pc_step}"

    def _get_step_tmpdir_name(self, pc_step):
        """Return the name of the interim folder for the process chain step
        Args:
            pc_step (int): The number of the step in the process chain where to
                           continue

        Returns:
            (str): The name of the interim folder for the process chain step

        """
        return f"tmpdir{pc_step}"

    def check_interim_result_mapset(self, pc_step, iteration):
        """Helper method to check if the interim result mapset is saved

        Args:
            pc_step (int): The number of the step in the process chain where to
                           continue

        Returns:
            interim_mapset (str): The path to the interim result mapset which
                                  has to be copied to the temporary mapsets
                                  (with _create_temporary_grass_environment)
            interim_file_path (str): The path to the interim result temporary
                                     file path which has to be copied to the
                                     temporary path
                                     (with _create_temporary_grass_environment)
        """
        iterim_error = False
        if self.saving_interim_results is False:
            iterim_error = True
            msg = "Saving iterim results is not configured"
        if os.path.isdir(self._get_interim_path()):
            interim_folder = os.listdir(self._get_interim_path())
        else:
            iterim_error = True
            msg = "No interim results saved in previous iteration"

        if iterim_error is False and (
            self._get_step_folder_name(pc_step) not in interim_folder
            or self._get_step_tmpdir_name(pc_step) not in interim_folder
        ):
            iterim_error = True
            msg = (
                "No interim results saved in previous iteration for "
                f"step {pc_step}"
            )
        if iterim_error is True:
            self.logger.error(msg)
            return None

        interim_mapset = self._get_interim_mapset_path(pc_step)
        interim_file_path = self._get_interim_tmpdir_path(pc_step)

        return interim_mapset, interim_file_path

    def _compare_sha512sums_of_folders(self, folder1, folder2):
        """Compares the sha512sums of two folders.
        Args:
            folder1 (str): Path to one folder
            folder2 (str): Path to another folder

        Returns:
            (bool): A boolean "True" if the sha512sums of the folder are the
                    same otherwise "False" (also if one of the folder does not
                    exist)
        """
        for folder in [folder1, folder2]:
            if not os.path.isdir(folder):
                self.logger.error(f"<{folder}> is no folder")
                return False

        # get sha512sum of the folder
        cur_working_dir = os.getcwd()
        sha512sum_cmd = (
            "find . -type f -exec sha512sum {} \; | " + "sort -k 2 | sha512sum"
        )
        sha512sums = list()
        for folder in [folder1, folder2]:
            os.chdir(folder)
            p_sha512sum_prev = os.popen(sha512sum_cmd)
            sha512sums.append(p_sha512sum_prev.read().strip())
        os.chdir(cur_working_dir)
        del cur_working_dir

        # compare sha512sums
        if sha512sums[0] == sha512sums[1]:
            return True
        else:
            return False

    def rsync_additional_mapsets(self, dest_path):
        """Using rsync to update additional mapsets from interim results to
        temporary mapset
        Args:
            dest_path (str): Path to destination folder where the additional
                             mapset should be saved
        """

        src_path = (
            f"{self._get_interim_mapset_path(self.old_pc_step)}_add_mapsets"
        )
        if not os.path.isdir(src_path):
            return

        for mapset in os.listdir(src_path):
            src = os.path.join(src_path, mapset)
            dest = os.path.join(dest_path, mapset)
            rsync_status = self.rsync_mapsets(src, dest)
            if rsync_status != "success":
                self.logger.info(
                    f"Syncing additional mapset <{mapset}> failed."
                )

    def rsync_mapsets(self, src, dest):
        """Using rsync to update the mapset folder.
        Args:
            src (str): Path of the source mapset
            dest (str): Path to destination mapset (where to rsync the src
                        mapset)

        Returns:
            (str): "success" if the rsync has worked otherwise "error"
        """
        rsync_cmd = [
            "rsync",
            "--recursive",
            "--compress",
            "--partial",
            "--progress",
            "--times",
            "--exclude",
            ".gislock",
            "--delete",
            src + os.sep,
            dest,
        ]
        p_rsync = subprocess.Popen(
            rsync_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        _, rsync_err = p_rsync.communicate()

        if rsync_err.decode("utf-8") != "":
            return "error"

        # remove .gislock
        gislock_file = os.path.join(dest, ".gislock")
        if os.path.isfile(gislock_file):
            os.remove(gislock_file)
        return "success"

    def _saving_folder(self, src, dest, old_dest, progress_step):
        """Saves the src folder to the dest folder by copying the directory or
        rsyncing it
        """
        # check if directory has changed
        same_dir = self._compare_sha512sums_of_folders(old_dest, src)
        if same_dir is True:
            self.logger.info(
                "Sha512sums of maspsets are equal; renaming interim result"
            )
            os.rename(old_dest, dest)
            return

        # get folder sizes in bytes
        size_prev_step = get_directory_size(old_dest)
        size_curr_step = get_directory_size(src)

        # rsync or copy folder
        if size_curr_step > size_prev_step * 0.9:
            self.logger.info("Copy old interim result")
            shutil.copytree(old_dest, dest)
        self.logger.info("Rsync mapset %s to interim result" % src)
        rsync_status = self.rsync_mapsets(src, dest)
        if rsync_status == "success":
            shutil.rmtree(old_dest)
        else:
            raise RsyncError("Error while rsyncing of step %d" % progress_step)

    def delete_interim_results(self):
        """Deletes the temporary mapset and temporary data"""
        interim_result_path = self._get_interim_path()

        if os.path.exists(interim_result_path) and os.path.isdir(
            interim_result_path
        ):
            shutil.rmtree(interim_result_path, ignore_errors=True)

    def _get_interim_path(self):
        """Returns the path where the interim results are saved"""
        return os.path.join(
            self.user_resource_interim_storage_path,
            self.resource_id,
        )

    def _get_included_additional_mapset_paths(
        self, temp_mapset_path, progress_step
    ):
        """Returns lists with source paths of hte additional mapsets and
        destination paths for them"""

        if self.include_additional_mapset_pattern:
            pattern = self.include_additional_mapset_pattern
            tmp_path = os.path.dirname(temp_mapset_path)
            dest_path = (
                f"{self._get_interim_mapset_path(progress_step)}_add_mapsets"
            )
            mapsets = filter(os.listdir(tmp_path), pattern)
            srcs = [os.path.join(tmp_path, mapset) for mapset in mapsets]
            dests = [os.path.join(dest_path, mapset) for mapset in mapsets]
            return srcs, dests
        else:
            return [], []

    def _get_interim_mapset_path(self, progress_step):
        """Returns path where the interim mapset is saved"""
        return os.path.join(
            self._get_interim_path(),
            self._get_step_folder_name(progress_step),
        )

    def _get_interim_tmpdir_path(self, progress_step=None):
        """Returns path where the interim directory is saved"""
        return os.path.join(
            self._get_interim_path(),
            self._get_step_tmpdir_name(progress_step),
        )

    def save_interim_results(
        self, progress_step, temp_mapset_path, temp_file_path, force_copy=False
    ):
        """Saves the temporary mapset to the
        `user_resource_interim_storage_path` by copying the directory or
        rsyncing it
        """

        # check if interim results should be saved for current endpoint
        if self.endpoint not in global_config.INTERIM_SAVING_ENDPOINTS:
            return

        if self.old_pc_step is not None:
            progress_step += self.old_pc_step
        self.logger.info("Saving interim results of step %d" % progress_step)
        dest_mapset = self._get_interim_mapset_path(progress_step)
        dest_tmpdir = self._get_interim_tmpdir_path(progress_step)
        addm_src, addm_dest = self._get_included_additional_mapset_paths(
            temp_mapset_path, progress_step
        )

        if temp_mapset_path is None:
            return

        # change mapset name for groups, raster VRTs and tgis
        for directory in ["group", "cell_misc", "tgis"]:
            change_mapsetname(
                os.path.join(temp_mapset_path, directory),
                directory,
                os.path.basename(temp_mapset_path),
                os.path.basename(dest_mapset),
            )

        if progress_step == 1 or force_copy is True:
            # copy temp mapset for first step
            shutil.copytree(temp_mapset_path, dest_mapset)
            shutil.copytree(temp_file_path, dest_tmpdir)
            self.logger.info(
                "Maspset %s and temp_file_path %s are copied"
                % (temp_mapset_path, temp_file_path)
            )
            for m_src, m_dest in zip(addm_src, addm_dest):
                shutil.copytree(m_src, m_dest)
        else:
            old_dest_mapset = self._get_interim_mapset_path(progress_step - 1)
            old_dest_tmpdir = self._get_interim_tmpdir_path(progress_step - 1)

            # saving mapset
            self._saving_folder(
                temp_mapset_path, dest_mapset, old_dest_mapset, progress_step
            )
            # saving temporary file path
            self._saving_folder(
                temp_file_path, dest_tmpdir, old_dest_tmpdir, progress_step
            )
            # saving additional mapsets
            _, old_dests = self._get_included_additional_mapset_paths(
                temp_mapset_path, progress_step - 1
            )
            for m_src, m_dest, old_dest in zip(addm_src, addm_dest, old_dests):
                self._saving_folder(m_src, m_dest, old_dest, progress_step)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Geodata processing commands
"""
import os
import requests
import zipfile
import magic
from urllib.request import urlopen
from urllib.parse import urlsplit
from actinia_core.core.common.exceptions import AsyncProcessError
from actinia_core.core.common.process_object import Process
from actinia_core.core.utils import get_wget_process, get_mv_process

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Julia Haas, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"

# Mimetypes supported for download
SUPPORTED_MIMETYPES = [
    "application/zip",
    "image/tiff",
    "application/gml",
    "text/xml",
    "application/x-sqlite3",
    "application/xml",
    "text/plain",
    "text/x-python",
]
# Suffixes supported in zip files
SUPPORTED_SUFFIXES = [
    ".tif",
    ".tiff",
    ".xml",
    ".gml",
    ".shp",
    ".dbf",
    ".shx",
    ".atx",
    ".sbx",
    ".qix",
    ".aih",
    ".prj",
    ".cpg",
    ".json",
]


class GeoDataDownloadImportSupport(object):
    """"""

    def __init__(
        self,
        config,
        temp_file_path,
        download_cache,
        send_resource_update,
        message_logger,
        url_list,
    ):
        """A collection of functions to generate geodata related import and
        processing commands. Each function returns a process chain that can be
        executed by the async processing classes.

        Args:
            config: The Actinia Core configuration object
            temp_file_path: The path to the temporary directory to store
                            temporary files. It is assumed that this path is
                            available when the generated commands are executed.
            download_cache (str): The path to the download cache
            send_resource_update: The function to call for resource updates
            message_logger: The message logger to be used
            url_list: A list of urls that should be accessed to download
                      imported geodata

        """
        self.config = config
        self.temp_file_path = temp_file_path
        self.user_download_cache_path = download_cache
        self.send_resource_update = send_resource_update
        self.message_logger = message_logger
        self.url_list = url_list
        self.detected_mime_types = []
        self.file_list = []
        self.copy_file_list = []
        self.import_file_info = []

    def _setup(self):
        """Setup the download cache.

        Check the download cache if the file already exists, to avoid redundant
        downloads. Create the cache if it does not exist and switch into the
        temporary directory.
        """

        # Create the download cache directory if it does not exists
        if os.path.exists(self.config.DOWNLOAD_CACHE):
            pass
        else:
            os.mkdir(self.config.DOWNLOAD_CACHE)

        # Create the user specific download cache directory to put the
        # downloaded files into it
        if os.path.exists(self.user_download_cache_path):
            pass
        else:
            os.mkdir(self.user_download_cache_path)

        # Change working directory to tempfile directory
        os.chdir(self.temp_file_path)

    def _check_urls(self):
        """Check the urls for access and supported mimetypes.
        If all files are already in the download cache, then
        nothing needs to be downloaded and checked.
        """
        for url in self.url_list:
            if url.startswith("/vsicurl/"):
                continue
            # Send a resource update
            if self.send_resource_update is not None:
                self.send_resource_update(
                    message="Checking access to URL: %s" % url
                )

            # Check if thr URL exists by investigating the HTTP header
            resp = requests.head(url)
            if self.message_logger:
                self.message_logger.info(
                    "%i %s %s" % (resp.status_code, resp.text, resp.headers)
                )

            if resp.status_code != 200:
                raise AsyncProcessError(
                    "The URL <%s> can not be accessed." % url
                )

            # Download 256 bytes from the url and check its mimetype
            response = urlopen(url)
            mime_type = magic.from_buffer(
                response.read(256), mime=True
            ).lower()
            if self.message_logger:
                self.message_logger.info(mime_type)

            if mime_type not in SUPPORTED_MIMETYPES:
                raise AsyncProcessError(
                    "Mimetype <%s> of url <%s> is not supported. "
                    "Supported mimetypes are: %s"
                    % (mime_type, url, ",".join(SUPPORTED_MIMETYPES))
                )

            self.detected_mime_types.append(mime_type)

    def get_download_process_list(self):
        """Create the process list to download, import and preprocess
        geodata location on a remote location

        The downloaded files will be stored in a temporary directory. After the
        download of all files completes, the downloaded files will be moved to
        the download cache. This avoids broken files in case a download was
        interrupted or stopped by termination.

        This method creates wget calls and mv calls.

        Returns:
            (download_commands, import_file_info)
        """
        self._setup()
        self._check_urls()

        download_commands = []
        count = 0
        create_copy_list = False

        if not self.copy_file_list:
            create_copy_list = True

        # Create the download commands and update process chain
        for url in self.url_list:
            # Extract file name from url and create temp and cache path
            # if the copy_file_path list is empty
            if create_copy_list is True:
                purl = urlsplit(url)
                file_name = os.path.basename(purl.path)
                source = os.path.join(self.temp_file_path, file_name)
                dest = os.path.join(self.user_download_cache_path, file_name)
                self.copy_file_list.append((source, dest))
            else:
                source, dest = self.copy_file_list[count]

            # Download file only if it does not exist in the download cache
            if os.path.isfile(dest) is False:
                p = get_wget_process(source, url)
                download_commands.append(p)
                if source != dest:
                    p = get_mv_process(source, dest)
                    download_commands.append(p)
            count += 1

        # Create the import file info list
        self.import_file_info = []

        for mtype, (source, dest) in zip(
            self.detected_mime_types, self.copy_file_list
        ):
            self.import_file_info.append((mtype, source, dest))

        return download_commands, self.import_file_info

    @staticmethod
    def get_file_rename_command(file_path, file_name):
        """Generate the file-rename process list so that the input file has a
        specific file name that is accessible in the process chain via file_id

        Args:
            file_path:
            file_name:

        Returns:
            Process

        """
        p = Process(
            exec_type="exec",
            executable="/bin/mv",
            executable_params=[file_path, file_name],
            id=f"mv_{os.path.basename(file_path)}_to_"
            f"{os.path.basename(file_name)}",
            skip_permission_check=True,
        )
        return p

    @staticmethod
    def get_raster_import_command(
        file_path,
        raster_name,
        resample=None,
        resolution=None,
        resolution_value=None,
    ):
        """Generate raster import process list that makes use of r.import

        Args:
            file_path (str): The path of the raster file to import
            raster_name (str): The name of the new raster layer
            resample (str): The resampling method to use for reprojection
                            Options: nearest, bilinear, bicubic, lanczos,
                            bilinear_f, bicubic_f, lanczos_f (default: nearest)
            resolution (str): The resolution of the new raster layer
                              Options: estimated, value, region (default:
                              estimated)
            resolution_value (str): Resolution of output raster map (use with
                                    option resolution=value). Must be in units
                                    of the target coordinate reference system,
                                    not in map units.

        Returns:
            Process

        """
        executable_params = [
            "input=%s" % file_path,
            "output=%s" % raster_name,
            "--q",
        ]

        if resample is not None:
            executable_params.append("resample=%s" % resample)
        if resolution is not None:
            executable_params.append("resolution=%s" % resolution)
        if resolution_value is not None:
            executable_params.append("resolution_value=%s" % resolution_value)

        p = Process(
            exec_type="grass",
            executable="r.import",
            executable_params=executable_params,
            id=f"r_import_{os.path.basename(file_path)}",
            skip_permission_check=True,
        )

        return p

    @staticmethod
    def get_vector_import_command(input_source, vector_name, layer_name=None):
        """Generate raster import process list that makes use of v.import

        Args:
            input_source (str): The input source can be a file path or a
                                database string
            vector_name (str): The name of the new vector layer
            layer_name (str): The layer name or comma separated list of layer
                              names that should be imported from the input
                              source

        Returns:
            Process

        """
        if layer_name is not None:
            exec_params = [
                "input=%s" % input_source,
                "output=%s" % vector_name,
                "layer=%s" % layer_name,
                "--q",
            ]
        else:
            exec_params = [
                "input=%s" % input_source,
                "output=%s" % vector_name,
                "--q",
            ]

        p = Process(
            exec_type="grass",
            executable="v.import",
            executable_params=exec_params,
            id=f"v_import_{os.path.basename(input_source)}",
            skip_permission_check=True,
        )
        return p

    def perform_file_validation(self, filepath, mimetype=None):
        """Perform a file validation check of mimetypes and zip bombs.
        This function checks zip files and returns the file names of the
        extracted file(s).
        If mimetype is None all supported mimetypes will be checked.

        Args:
            filepath (str): The path to a file that should be checked against
                      supported mimetypes and zip-bomb security.
            mimetype (str): A specific mimetype that should be checked

        Returns:
            (list)
            file_list A list of files that will be in the current working
                      directory
        """
        file_name = os.path.basename(filepath)
        file_list = [file_name]

        if not os.path.isfile(filepath):
            raise AsyncProcessError("File <%s> does not exist." % filepath)

        mime_type = magic.from_file(filepath, mime=True)
        if self.message_logger:
            self.message_logger.info(mime_type)

        if mime_type not in SUPPORTED_MIMETYPES:
            raise AsyncProcessError(
                "Mimetype of url <%s> is not supported. "
                "Supported mimetypes are: %s"
                % (filepath, ",".join(SUPPORTED_MIMETYPES))
            )

        if mime_type.lower() == "application/zip":
            z = zipfile.ZipFile(filepath)
            total_sum = sum(e.file_size for e in z.infolist())
            compressed_sum = sum(e.compress_size for e in z.infolist())
            compression_ratio = total_sum / compressed_sum

            print(compressed_sum, total_sum, compression_ratio)

            if compression_ratio > 10000:
                raise AsyncProcessError(
                    "Compression ratio is larger than 10000."
                )

            if total_sum > 2**32:
                raise AsyncProcessError(
                    "Files larger than 4GB are not supported in zip files."
                )

            for name in z.namelist():
                file_name, suffix = os.path.splitext(name)
                file_list.append(file_name)
                if suffix not in SUPPORTED_SUFFIXES:
                    raise AsyncProcessError(
                        "Suffix %s of zipped file <%s> is not "
                        "supported. Supported suffixes in zip "
                        "files are: %s"
                        % (suffix, name, ",".join(SUPPORTED_SUFFIXES))
                    )
            z.close()

        return file_list



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: User test case
"""
import unittest
from actinia_core.core.common.user import ActiniaUser
from actinia_core.core.common.app import flask_app

try:
    from .test_resource_base import ActiniaResourceTestCaseBase
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class UserTestCase(ActiniaResourceTestCaseBase):
    """
    This class tests the user interface, the creation and deletion
    of a user entry in a Redis database
    """

    def setUp(self):
        # We need to set the application context
        self.app_context = flask_app.app_context()
        self.app_context.push()

        # Create a test user
        self.user_id = "soeren"
        self.password = "1234"
        self.user_group = "test"

    def tearDown(self):
        self.app_context.pop()

    def test_create_delete_user(self):
        # Make sure the user database is empty
        user = ActiniaUser(self.user_id)
        if user.exists():
            print("Delete existing user")
            user.delete()

        # Create a new user
        user = ActiniaUser.create_user(
            self.user_id,
            self.user_group,
            self.password,
            user_role="admin",
            accessible_datasets={"nc_spm_08": ["PERMANENT", "user1"]},
            accessible_modules=["g.region", "g.mapset", "r.slope.aspect"],
            cell_limit=1000,
            process_num_limit=3,
            process_time_limit=30,
        )

        print(user)

        self.assertEqual(user.get_role(), "admin")
        self.assertEqual(user.get_id(), self.user_id)
        self.assertEqual(user.get_group(), self.user_group)
        self.assertEqual(user.get_cell_limit(), 1000)
        self.assertEqual(user.get_process_num_limit(), 3)
        self.assertEqual(user.get_process_time_limit(), 30)

        datasets = user.get_accessible_datasets()
        modules = user.get_accessible_modules()

        self.assertTrue("nc_spm_08" in datasets)
        self.assertTrue("PERMANENT" in datasets["nc_spm_08"])
        self.assertTrue("g.region" in modules)

        token = user.generate_auth_token()
        user_2 = ActiniaUser.verify_auth_token(token)

        print(user_2)

        self.assertTrue(user_2.exists())
        self.assertEqual(user_2.get_role(), "admin")
        self.assertEqual(user_2.get_id(), self.user_id)
        self.assertEqual(user.get_group(), self.user_group)
        self.assertEqual(user_2.get_cell_limit(), 1000)
        self.assertEqual(user_2.get_process_num_limit(), 3)
        self.assertEqual(user_2.get_process_time_limit(), 30)

        datasets = user_2.get_accessible_datasets()
        modules = user_2.get_accessible_modules()

        self.assertTrue("nc_spm_08" in datasets)
        self.assertTrue("PERMANENT" in datasets["nc_spm_08"])
        self.assertTrue("g.region" in modules)

        api_key = user.generate_api_key()
        user_3 = ActiniaUser.verify_api_key(api_key)

        print(user_3)

        self.assertTrue(user_3.exists())
        self.assertEqual(user_3.get_role(), "admin")
        self.assertEqual(user_3.get_id(), self.user_id)
        self.assertEqual(user.get_group(), self.user_group)
        self.assertEqual(user_3.get_cell_limit(), 1000)
        self.assertEqual(user_3.get_process_num_limit(), 3)
        self.assertEqual(user_3.get_process_time_limit(), 30)

        datasets = user_3.get_accessible_datasets()
        modules = user_3.get_accessible_modules()

        self.assertTrue("nc_spm_08" in datasets)
        self.assertTrue("PERMANENT" in datasets["nc_spm_08"])
        self.assertTrue("g.region" in modules)

        self.assertTrue(user.delete())
        self.assertFalse(user_2.delete())
        self.assertFalse(user_3.delete())

    def test_create_update_user(self):
        """Test the creation and update of a user in the redis database"""

        user = ActiniaUser(self.user_id)
        if user.exists():
            print("Delete existing user")
            user.delete()

        # Create a new user
        user = ActiniaUser.create_user(
            self.user_id,
            self.user_group,
            self.password,
            user_role="admin",
            accessible_datasets={"nc_spm_08": ["PERMANENT", "user1"]},
            accessible_modules=["g.region", "g.mapset", "r.slope.aspect"],
            cell_limit=1000,
            process_num_limit=3,
            process_time_limit=30,
        )

        print(user)

        self.assertEqual(user.get_role(), "admin")
        self.assertEqual(user.get_id(), self.user_id)
        self.assertEqual(user.get_group(), self.user_group)
        self.assertEqual(user.get_cell_limit(), 1000)
        self.assertEqual(user.get_process_num_limit(), 3)
        self.assertEqual(user.get_process_time_limit(), 30)

        datasets = user.get_accessible_datasets()

        self.assertTrue("nc_spm_08" in datasets)
        self.assertTrue("PERMANENT" in datasets["nc_spm_08"])

        user = ActiniaUser(user_id=self.user_id)
        user.read_from_db()
        user.set_role("user")
        user.set_cell_limit(1000000)
        user.set_process_num_limit(10)
        user.set_process_time_limit(50)
        user.update()

        print(user)

        self.assertEqual(user.get_role(), "user")
        self.assertEqual(user.get_cell_limit(), 1000000)
        self.assertEqual(user.get_process_num_limit(), 10)
        self.assertEqual(user.get_process_time_limit(), 50)

        user = ActiniaUser(user_id=self.user_id)
        user.read_from_db()
        user.add_accessible_dataset("utm32n", ["PERMANENT"])
        user.add_accessible_modules(
            [
                "i.vi",
            ]
        )
        user.update()

        print(user)

        datasets = user.get_accessible_datasets()
        modules = user.get_accessible_modules()

        self.assertTrue("nc_spm_08" in datasets)
        self.assertTrue("PERMANENT" in datasets["nc_spm_08"])
        self.assertTrue("g.region" in modules)
        self.assertTrue("utm32n" in datasets)
        self.assertTrue("PERMANENT" in datasets["utm32n"])
        self.assertTrue("i.vi" in modules)

        user = ActiniaUser(user_id=self.user_id)
        user.read_from_db()
        user.remove_location("utm32n")
        user.remove_mapsets_from_location(
            "nc_spm_08",
            [
                "user1",
            ],
        )
        user.remove_accessible_modules(
            [
                "i.vi",
            ]
        )
        user.update()

        print(user)

        datasets = user.get_accessible_datasets()
        modules = user.get_accessible_modules()

        self.assertTrue("nc_spm_08" in datasets)
        self.assertTrue("PERMANENT" in datasets["nc_spm_08"])
        self.assertFalse("user1" in datasets["nc_spm_08"])
        self.assertTrue("g.region" in modules)
        self.assertFalse("utm32n" in datasets)
        self.assertFalse("i.vi" in modules)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async processing PostGIS test case
"""
import unittest

# from flask.json import dumps as json_dumps
from random import randint

try:
    from .test_resource_base import ActiniaResourceTestCaseBase  # , URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase  # , URL_PREFIX

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

process_chain_postgis = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "PG:dbname=gis user=postgres host=localhost "
                        "port=5432 password=postgres",
                        "type": "postgis",
                        "vector_layer": "poly",
                    },
                    "param": "map",
                    "value": "poly",
                }
            ],
        },
        {
            "id": "exporter_1",
            "module": "exporter",
            "outputs": [
                {
                    "export": {
                        "dbstring": "PG:dbname=gis user=postgres "
                        "host=localhost port=5432 password=postgres",
                        "format": "PostgreSQL",
                        "type": "vector",
                        "output_layer": "poly_2",
                    },
                    "param": "map",
                    "value": "poly",
                }
            ],
        },
    ],
    "webhooks": {
        "finished": "http://0.0.0.0:5005/webhook/finished",
        "update": "http://0.0.0.0:5005/webhook/update",
    },
    "version": "1",
}


class AsyncProcessingPostGISTestCase(ActiniaResourceTestCaseBase):
    def gen_output_layer_name(self):
        process_chain_postgis["list"][1]["outputs"][0]["export"][
            "output_layer"
        ] = "poly_%i" % randint(0, 1000000000)

    # # TODO fix test and comment the test in (postgres DB is needed)
    # def test_1_async_processing_postgis_validation(self):
    #     rv = self.server.post(
    #         f"{URL_PREFIX}/locations/nc_spm_08/process_chain_validation_async",
    #         headers=self.admin_auth_header,
    #         data=json_dumps(process_chain_postgis),
    #         content_type="application/json",
    #     )
    #
    #     resp = self.waitAsyncStatusAssertHTTP(
    #         rv,
    #         headers=self.admin_auth_header,
    #         http_status=200,
    #         status="finished",
    #     )
    #     self.assertEqual(len(resp["process_results"]), 1)
    #
    # # TODO fix test and comment the test in (postgres DB is needed)
    # def test_2_async_processing_postgis_run(self):
    #     # Don't overwrite an existing layer
    #     self.gen_output_layer_name()
    #
    #     rv = self.server.post(
    #         URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
    #         headers=self.admin_auth_header,
    #         data=json_dumps(process_chain_postgis),
    #         content_type="application/json",
    #     )
    #
    #     resp = self.waitAsyncStatusAssertHTTP(
    #         rv,
    #         headers=self.admin_auth_header,
    #         http_status=200,
    #         status="finished",
    #     )
    #     self.assertEqual(len(resp["process_log"]), 2)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async process validation test case
"""
import unittest
from flask.json import dumps as json_dumps, loads as json_loads
from pprint import pprint

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

# param wrong
process_chain_error_1 = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [
                {"paramp": "raster", "value": "elevation@PERMANENT"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        }
    ],
}

# value wrong
process_chain_error_2 = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "values": "elevation@PERMANENT"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        }
    ],
}

# module wrong
process_chain_error_3 = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "modules": "g.region",
            "inputs": [
                {"param": "raster", "value": "elevation@PERMANENT"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        }
    ],
}

# Id wrong
process_chain_error_4 = {
    "version": 1,
    "list": [
        {
            "ids": "1",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "value": "elevation@PERMANENT"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        }
    ],
}

# Wrong mapset
process_chain_error_5 = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "value": "elevation@PERMAFROST"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        }
    ],
}

# param missing on output
process_chain_output_error_1 = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "value": "elevation@PERMANENT"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        },
        {
            "id": "2",
            "module": "r.out.ascii",
            "inputs": [{"param": "input", "value": "elevation@PERMANENT"}],
            "outputs": [{"params": "output", "value": "$file::out1"}],
        },
    ],
}

# value missing on output
process_chain_output_error_2 = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "value": "elevation@PERMANENT"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        },
        {
            "id": "2",
            "module": "r.out.ascii",
            "inputs": [{"param": "input", "value": "elevation@PERMANENT"}],
            "outputs": [{"param": "output", "values": "$file::out1"}],
        },
    ],
}

# Wrong Landsat scene id
process_chain_landsat_error_1 = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "LT52170762005240COA00-NOT",
                        "type": "landsat",
                        "landsat_atcor": "dos1",
                    },
                    "param": "map",
                    "value": "ignored",
                }
            ],
        }
    ],
    "version": "1",
}

# Wrong type
process_chain_landsat_error_2 = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "LT52170762005240COA00",
                        "type": "landsatpo",
                        "landsat_atcor": "dos1",
                    },
                    "param": "map",
                    "value": "ignored",
                }
            ],
        }
    ],
    "version": "1",
}

# Wrong Landsat atcor type
process_chain_landsat_error_3 = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "LT52170762005240COA00",
                        "type": "landsat",
                        "landsat_atcor": "ddos1",
                    },
                    "param": "map",
                    "value": "ignored",
                }
            ],
        }
    ],
    "version": "1",
}

# Missing sentinel_band
process_chain_sent_1 = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "S2B_MSIL1C_20171010T131249_N0205_R081_"
                        "T26VPR_20171010T131243",
                        "type": "sentinel2",
                    },
                    "param": "map",
                    "value": "B04",
                }
            ],
        }
    ],
    "version": "1",
}

# Missing type
process_chain_sent_2 = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "S2B_MSIL1C_20171010T131249_N0205_R081_"
                        "T26VPR_20171010T131243",
                        "sentinel_band": "B04",
                    },
                    "param": "map",
                    "value": "B04",
                }
            ],
        }
    ],
    "version": "1",
}


# Wrong webhook URL
process_chain_error_webhook_finished = {
    "version": 1,
    "webhooks": {
        "finished": "http://0.0.0.0:5005/webhook/finished_not",
        "update": "http://0.0.0.0:5005/webhook/update",
    },
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "value": "elevation@PERMANENT"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        }
    ],
}


process_chain_error_webhook_update = {
    "version": 1,
    "webhooks": {
        "finished": "http://0.0.0.0:5005/webhook/finished",
        "update": "http://0.0.0.0:5005/webhook/update_not",
    },
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "value": "elevation@PERMANENT"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        }
    ],
}


class AsyncProcessValidationTestCase(ActiniaResourceTestCaseBase):
    def test_async_processing_error_webhook_finished(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_error_webhook_finished),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_error_webhook_update(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_error_webhook_update),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_error_1(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_error_1),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_error_2(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_error_2),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_error_3(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_error_3),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_error_4(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_error_4),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_error_5(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_error_5),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_output_error_1(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_output_error_1),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_output_error_2(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_output_error_2),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_landsat_error_1(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_landsat_error_1),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_landsat_error_2(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_landsat_error_2),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_landsat_error_3(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_landsat_error_3),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_sent_error_1(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_sent_1),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_sent_error_2(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_sent_2),
            content_type="application/json",
        )

        pprint(json_loads(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Resource Data Container
"""
from .storage_interface_filesystem import ResourceStorageFilesystem
from .storage_interface_aws_s3 import ResourceStorageS3
from .storage_interface_gcs import ResourceStorageGCS

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class ResourceDataContainer(object):
    """Class that manages the data send from the resource
    to the enqueued asynchronous processing object
    """

    def __init__(
        self,
        grass_data_base,
        grass_user_data_base,
        grass_base_dir,
        request_data,
        user_id,
        user_group,
        resource_id,
        iteration,
        status_url,
        api_info,
        resource_url_base,
        orig_time,
        orig_datetime,
        user_credentials,
        config,
        location_name,
        mapset_name,
        map_name,
    ):
        """

        Args:
            grass_data_base (str): GRASS GIS database root directory that
                                   contains global locations
            grass_user_data_base (str): GRASS GIS database user directory that
                                        contains group specific locations
            grass_base_dir (str): The installation directory of GRASS GIS
            request_data (dict): The module chain dictionary
            user_id (str): The user id
            user_group (str): The user group
            resource_id (str): The id of the resource
            iteration (int): The iteartion of the job
            status_url (str): The url to be used to create the status response
            api_info (ApiInfoModel): Information about the API call
            resource_url_base (str): The base URL for resources
            orig_time (time): The time of origin (seconds)
            orig_datetime (datetime): The datetime of origin (datetime format)
            user_credentials (dict): The user credentials dict
            config (actinia_core.core.config.Configuration): The actinia
                                                             configuration
            location_name (str): The name of the location to work in
            mapset_name (str): The name of the target mapset in which the
                               computation should be performed
            map_name: The name of the map or other resource (raster, vector,
                      STRDS, color, ...)

        """

        self.grass_data_base = grass_data_base
        self.grass_user_data_base = grass_user_data_base
        self.grass_base_dir = grass_base_dir
        self.request_data = request_data
        self.user_id = user_id
        self.user_group = user_group
        self.resource_id = resource_id
        self.iteration = iteration
        self.status_url = status_url
        self.api_info = api_info
        self.resource_url_base = resource_url_base
        self.user_credentials = user_credentials
        self.config = config
        self.location_name = location_name
        self.mapset_name = mapset_name
        self.map_name = map_name
        self.orig_time = orig_time
        self.orig_datetime = orig_datetime
        self.user_data = None
        self.storage_model = "file"
        self.queue = None

    # def __str__(self):
    #    return str(self.__dict__)

    def set_user_data(self, user_data):
        """Put all required data for processing into the data object

        Args:
            user_data: Any payload that must be passed in addition tot the
                       request data, like command arguments and so on

        """
        self.user_data = user_data

    def set_request_data(self, request_data):
        self.request_data = request_data

    def set_queue_name(self, queue_name):
        self.queue = queue_name

    def set_storage_model_to_file(self):
        self.storage_model = "file"

    def set_storage_model_to_s3(self):
        self.storage_model = "s3"

    def set_storage_model_to_gcs(self):
        self.storage_model = "gcs"

    def get_storage_model(self):
        return self.storage_model

    def is_storage_model_file(self):
        return self.storage_model == "file"

    def is_storage_model_s3(self):
        return self.storage_model == "s3"

    def is_storage_model_gcs(self):
        return self.storage_model == "gcs"

    def create_storage_interface(self):
        """Create the storage interface

        Returns: Either ResourceStorageFilesystem, ResourceStorageGCS or
                 ResourceStorageS3
        """
        if self.is_storage_model_file():
            return ResourceStorageFilesystem(
                user_id=self.user_id,
                resource_id=self.resource_id,
                config=self.config,
                resource_url_base=self.resource_url_base,
            )

        if self.is_storage_model_s3():
            return ResourceStorageS3(
                user_id=self.user_id,
                resource_id=self.resource_id,
                config=self.config,
            )

        if self.is_storage_model_gcs():
            return ResourceStorageGCS(
                user_id=self.user_id,
                resource_id=self.resource_id,
                config=self.config,
            )



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Request parser
"""

from flask_restful import reqparse

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


# Create a temporal module where, order, column parser
where_parser = reqparse.RequestParser()
where_parser.add_argument(
    "where", type=str, location="args", help="A SQL where option"
)
where_parser.add_argument(
    "order", type=str, location="args", help="The order of the listing"
)
where_parser.add_argument(
    "columns", type=str, location="args", help="The columns to be listed"
)


def extract_where_parameters(args):
    """Parse the request parameters for modules that have where, order and
    column options from parsed arguments that were created with the
    where_parser.

    Args:
        args (dict): The argument dictionary with where option

    Returns:
         list:
         A list of where and/or t_where parameter
    """
    options = []
    if "where" in args and args["where"] is not None:
        options.append("where=%s" % args["where"])
    if "order" in args and args["order"] is not None:
        options.append("order=%s" % args["order"])
    if "columns" in args and args["columns"] is not None:
        options.append("columns=%s" % args["columns"])

    return options


# Create a g.list/g.remove pattern parser
glist_parser = reqparse.RequestParser()
glist_parser.add_argument(
    "pattern",
    type=str,
    help="A pattern string must be provided",
    location="args",
)


def extract_glist_parameters(args):
    """Parse the request parameters for g.list/g.remove from parsed arguments
    that were created with the glist_parser.

    Args:
        args (dict): The argument dictionary with g.list/g.remove pattern
                     option

    Returns:
         list:
         A dict of g.list parameter
    """
    options = {}

    if "pattern" in args and args["pattern"] is not None:
        options["pattern"] = args["pattern"]

    return options


# Create a grass EPSG code parser
start_script_parser = reqparse.RequestParser()
start_script_parser.add_argument(
    "epsg",
    required=True,
    location="args",
    type=int,
    help="A EPSG code as integer must be provided",
)


def extract_start_script_parameters(args):
    """Parse the request parameters for the GRASS start script from parsed
    arguments that were created with the start_script_parser.

    Args:
        args (dict): The argument dictionary with grass start script options

    Returns:
         list:
         A list of grass start script parameters
    """
    options = []
    if "epsg" in args:
        options.append("-c")
        options.append("EPSG:%i" % args["epsg"])

    return options


# Create a t.create option parser
t_create_parser = reqparse.RequestParser()
t_create_parser.add_argument(
    "temporaltype",
    type=str,
    location="args",
    help="The temporal type of the space time dataset,"
    "options: absolute,relative, default: absolute",
)
t_create_parser.add_argument(
    "title",
    required=True,
    type=str,
    location="args",
    help="Title of the new space time raster dataset",
)
t_create_parser.add_argument(
    "description",
    required=True,
    type=str,
    location="args",
    help="Description of the new space time dataset",
)


def extract_t_create_parameters(args):
    """Parse the request parameters for t.create

    Args:
        args (dict): The argument dictionary with where option

    Returns:
         list:
         A list of t.create parameter
    """
    options = []
    if "temporaltype" in args and args["temporaltype"] is not None:
        options.append("temporaltype=%s" % args["temporaltype"])
    if "title" in args and args["title"] is not None:
        options.append("title=%s" % args["title"])
    if "description" in args and args["description"] is not None:
        options.append("description=%s" % args["description"])
    options.append("semantictype=mean")

    return options


# Create a t.register option parser
t_register_parser = reqparse.RequestParser()
t_register_parser.add_argument(
    "start",
    type=str,
    location="args",
    help="Valid start date and time of the first map. "
    'Format absolute time: "yyyy-mm-dd HH:MM:SS '
    '+HHMM", relative time is of type integer',
)
t_register_parser.add_argument(
    "end",
    type=str,
    location="args",
    help="Valid end date and time of all maps. "
    'Format absolute time: "yyyy-mm-dd HH:MM:SS0'
    ' +HHMM", relative time is of type integer',
)
t_register_parser.add_argument(
    "unit",
    type=str,
    location="args",
    help="Time stamp unit. Unit must be set in case of "
    "relative time stamps. options: "
    "years,months,days,hours,minutes,seconds",
)
t_register_parser.add_argument(
    "increment",
    type=str,
    location="args",
    help="Time increment, works only in conjunction with "
    "start option. Time increment between maps for "
    "valid time interval creation (format absolute: "
    "NNN seconds, minutes, hours, days, weeks, "
    "months, years; format relative is integer: 5)",
)


def extract_t_register_parameters(args):
    """Parse the request parameters for t.register

    Args:
        args (dict): The argument dictionary with where option

    Returns:
         list:
         A list of t.register parameter
    """
    options = []
    if "start" in args and args["start"] is not None:
        options.append("start=%s" % args["start"])
    if "end" in args and args["end"] is not None:
        options.append("end=%s" % args["end"])
    if "unit" in args and args["unit"] is not None:
        options.append("unit=%s" % args["unit"])
    if "increment" in args and args["increment"] is not None:
        options.append("increment=%s" % args["increment"])

    return options



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
This module takes care of the status requests that are generated by
asynchronous processes.
"""

import pickle
import re
from flask import (
    g,
    request,
)
from flask import jsonify, make_response
from flask_restful_swagger_2 import Resource
from flask_restful_swagger_2 import swagger
from flask_restful import reqparse
from time import sleep
from actinia_api.swagger2.actinia_core.apidocs import resource_management

from actinia_core.core.common.app import auth
from actinia_core.core.common.config import global_config, DEFAULT_CONFIG_PATH
from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.core.resources_logger import ResourceLogger
from actinia_core.core.common.api_logger import log_api_call
from actinia_core.core.common.user import ActiniaUser
from actinia_core.models.response_models import (
    SimpleResponseModel,
    ProcessingResponseListModel,
)
from actinia_core.core.interim_results import InterimResult

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis GmbH & Co. KG"


class ResourceManagerBase(Resource):
    """Base class for resource management"""

    decorators = [log_api_call, auth.login_required]

    def __init__(self):
        # Configuration
        Resource.__init__(self)

        kwargs = dict()
        kwargs["host"] = global_config.REDIS_SERVER_URL
        kwargs["port"] = global_config.REDIS_SERVER_PORT
        if (
            global_config.REDIS_SERVER_PW
            and global_config.REDIS_SERVER_PW is not None
        ):
            kwargs["password"] = global_config.REDIS_SERVER_PW
        self.resource_logger = ResourceLogger(**kwargs)
        del kwargs

        # Store the user id, user group and all credentials of the current user

        self.user = g.user
        self.user_id = g.user.get_id()
        self.user_group = g.user.get_group()
        self.user_role = g.user.get_role()
        self.user_credentials = g.user.get_credentials()

    def check_permissions(self, user_id):
        """Check the access rights of the user that calls this API call

        Permission:
            - guest and user roles can only access resources of the same user
              id
            - admin role are allowed to access resources of users with the same
              user group, except for superusers
            - superdamins role can access all resources

        Args:
            user_id:

        Returns:
            None if permissions granted, a error response if permissions are
            not fulfilled

        """
        # Superuser are allowed to do everything
        if self.user.has_superadmin_role() is True:
            return None

        # Check permissions for users and guests
        if self.user_role == "guest" or self.user_role == "user":
            if self.user_id != user_id:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="You do not have the permission to access "
                            "this resource. Wrong user.",
                        )
                    ),
                    401,
                )

        if global_config.KEYCLOAK_CONFIG_PATH and self.user_role == "admin":
            if self.user.check_group_members(user_id):
                return None
            else:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="You do not have the permission to access "
                            "this resource. Wrong user group. (Maybe you can c"
                            "heck the group_members in the group attributes.)",
                        )
                    ),
                    401,
                )

        new_user = ActiniaUser(user_id=user_id)

        # Check if the user exists
        if new_user.exists() != 1:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error",
                        message="The user <%s> does not exist" % user_id,
                    )
                ),
                400,
            )

        # Check admin permissions
        if self.user_role == "admin":
            # Resources of superusers are not allowed to be accessed
            if new_user.has_superadmin_role() is True:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="You do not have the permission to access "
                            "this resource. Wrong user role.",
                        )
                    ),
                    401,
                )
            # Only resources of the same user group are allowed to be accessed
            if new_user.get_group() != self.user_group:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="You do not have the permission to access "
                            "this resource. Wrong user group.",
                        )
                    ),
                    401,
                )
        return None


class ResourceManager(ResourceManagerBase):
    """
    This class is responsible to answer status requests
    of asynchronous processes (resources) and
    to request the termination of a resource
    """

    def __init__(self):
        # Configuration
        ResourceManagerBase.__init__(self)

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", resource_management.resource_get_doc))
    def get(self, user_id, resource_id):
        """Get the status of a resource."""

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        # the latest iteration should be given
        if resource_id.startswith("resource_id-"):
            _, response_data = self.resource_logger.get_latest_iteration(
                user_id, resource_id
            )
        else:
            response_data = self.resource_logger.get_all_iteration(
                user_id, "resource_id-%s" % resource_id
            )

        if response_data is not None:
            http_code, response_model = pickle.loads(response_data)
            return make_response(jsonify(response_model), http_code)
        else:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource does not exist"
                    )
                ),
                400,
            )

    def _check_possibility_of_new_iteration(
        self, response_model, user_id, resource_id
    ):
        """Check if it possible to start a new iteration of the process chain
        of the resource_id. A new iteration is only possible if the status of
        the resource is error or terminated, or running but the time_delta does
        not change any more.

        Args:
            response_model (ProcessingResponseModel): The processing response
                                                      model of the old
                                                      iteration of the resource
            user_id (str): The unique user name/id
            resource_id (str): The id of the resource
        """
        error_msg = None
        if response_model is None:
            error_msg = "Resource has no response model"
        if response_model["status"] in ["accepted", "finished"]:
            error_msg = (
                f"Resource is {response_model['status']} PUT not possible"
            )
        elif response_model["status"] in ["running"]:
            sleep(5)
            _, response_data2 = self.resource_logger.get_latest_iteration(
                user_id, resource_id
            )
            if response_data2 is None:
                error_msg = "Resource does not exist"
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error", message="Resource does not exist"
                        )
                    ),
                    400,
                )
            _, response_model2 = pickle.loads(response_data2)
            if response_model2 is None:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="Resource has no response model",
                        )
                    ),
                    400,
                )
            # check time_delta and set not running processes to error
            if response_model["time_delta"] == response_model2["time_delta"]:
                iteration = (
                    response_model2["iteration"]
                    if "iteration" in response_model2
                    else None
                )
                response_model2["status"] = "error"
                response_model2["message"] = (
                    "The process no longer seems to be "
                    "running and has therefore been set to error."
                )
                redis_return = self.resource_logger.commit(
                    user_id,
                    resource_id,
                    iteration,
                    pickle.dumps([200, response_model2]),
                )
                if redis_return is True:
                    pass
                else:
                    error_msg = (
                        "Resource is running and can not be set to error"
                    )
            else:
                error_msg = "Resource is running no restart possible"
        elif response_model["status"] in ["error", "terminated"]:
            pass
        return error_msg

    def _create_ResourceDataContainer_for_resumption(
        self,
        post_url,
        pc_step,
        user_id,
        resource_id,
        iteration,
        endpoint,
        process_chain_list=None,
    ):
        """Create the ResourceDataContainer for the resumption of the resource
        depending on the post_url

        Args:
            post_url (str): The request url of the original resource
            pc_step (int): The number of the process chain step where to
                           continue the process
            user_id (str): The unique user name/id
            resource_id (str): The id of the resource
            iteration (int): The number of iteration of this resource
            process_chain_list (dict): The process chain list (e.g. for the
                                       job resumption when no new postbody is
                                       send in the PUT request)

        Returns:
            rdc (ResourceDataContainer): The data container that contains all
                                         required variables for processing
            processing_resource (AsyncEphemeralResource/AsyncPersistentResource
            /AsyncEphemeralExportResource): The processing resource
            start_job (function): The start job function of the
                                  processing_resource
        """
        preprocess_kwargs = {}
        if process_chain_list is not None:
            preprocess_kwargs["has_json"] = False
            preprocess_kwargs["process_chain_list"] = process_chain_list

        interim_result = InterimResult(
            user_id, resource_id, iteration, endpoint
        )
        if (
            interim_result.check_interim_result_mapset(pc_step, iteration - 1)
            is None
        ):
            return None, None, None
        location = re.findall(r"locations\/(.*?)\/", post_url)[0]
        processing_class = global_config.INTERIM_SAVING_ENDPOINTS[endpoint]
        if processing_class == "AsyncEphemeralResource":
            # /locations/<string:location_name>/processing_async
            from .ephemeral_processing import AsyncEphemeralResource
            from ..processing.common.ephemeral_processing import start_job

            processing_resource = AsyncEphemeralResource(
                resource_id, iteration, post_url
            )
            rdc = processing_resource.preprocess(
                location_name=location, **preprocess_kwargs
            )
        elif processing_class == "AsyncPersistentResource":
            # /locations/{location_name}/mapsets/{mapset_name}/processing_async
            from .persistent_processing import AsyncPersistentResource
            from ..processing.common.persistent_processing import start_job

            processing_resource = AsyncPersistentResource(
                resource_id, iteration, post_url
            )
            mapset = re.findall(r"mapsets\/(.*?)\/", post_url)[0]
            rdc = processing_resource.preprocess(
                location_name=location, mapset_name=mapset, **preprocess_kwargs
            )

        elif processing_class == "AsyncEphemeralExportResource":
            # /locations/{location_name}/processing_async_export
            from .ephemeral_processing_with_export import (
                AsyncEphemeralExportResource,
            )
            from ..processing.common.ephemeral_processing_with_export import (
                start_job,
            )

            processing_resource = AsyncEphemeralExportResource(
                resource_id, iteration, post_url
            )
            rdc = processing_resource.preprocess(
                location_name=location, **preprocess_kwargs
            )
        else:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error",
                        message=f"Processing endpoint {post_url} does not "
                        "support put",
                    )
                ),
                400,
            )
        rdc.api_info["endpoint"] = endpoint
        return rdc, processing_resource, start_job

    @endpoint_decorator()
    @swagger.doc(check_endpoint("put", resource_management.resource_put_doc))
    def put(self, user_id, resource_id):
        """Updates/Resumes the status of a resource."""
        global_config.read(DEFAULT_CONFIG_PATH)
        if (
            global_config.SAVE_INTERIM_RESULTS is not True
            and global_config.SAVE_INTERIM_RESULTS != "onError"
        ):
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error",
                        message="Interim results are not set in the "
                        "configuration",
                    )
                ),
                404,
            )

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        # check if latest iteration is found
        (
            old_iteration,
            response_data,
        ) = self.resource_logger.get_latest_iteration(user_id, resource_id)
        old_iteration = 1 if old_iteration is None else old_iteration
        if response_data is None:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource does not exist"
                    )
                ),
                400,
            )

        # check if a new iteration is possible
        _, response_model = pickle.loads(response_data)
        err_msg = self._check_possibility_of_new_iteration(
            response_model, user_id, resource_id
        )
        if err_msg is not None:
            return make_response(
                jsonify(SimpleResponseModel(status="error", message=err_msg)),
                404,
            )
        # get step of the process chain
        pc_step = response_model["progress"]["step"] - 1
        for iter in range(old_iteration - 1, 0, -1):
            if iter == 1:
                old_response_data = self.resource_logger.get(
                    user_id, resource_id
                )
            else:
                old_response_data = self.resource_logger.get(
                    user_id, resource_id, iter
                )
            if old_response_data is None:
                return None
            _, old_response_model = pickle.loads(old_response_data)
            pc_step += old_response_model["progress"]["step"] - 1

        # start new iteration
        iteration = old_iteration + 1

        # use post_url if iteration > 1
        if old_iteration and old_iteration == 1:
            post_url = response_model["api_info"]["request_url"]
        elif old_iteration and old_iteration > 1:
            post_url = response_model["api_info"]["post_url"]
        else:
            post_url = None

        # use old process chain list if no new one is send
        process_chain_list = None
        if not request.is_json:
            process_chain_list = response_model["process_chain_list"][0]

        rdc_resp = self._create_ResourceDataContainer_for_resumption(
            post_url,
            pc_step,
            user_id,
            resource_id,
            iteration,
            response_model["api_info"]["endpoint"],
            process_chain_list=process_chain_list,
        )

        if len(rdc_resp) == 3:
            rdc = rdc_resp[0]
            processing_resource = rdc_resp[1]
            start_job = rdc_resp[2]
        else:
            return rdc_resp

        # enqueue job
        if rdc:
            enqueue_job(processing_resource.job_timeout, start_job, rdc)
        else:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource cannot be resumed."
                    )
                ),
                400,
            )
        html_code, response_model = pickle.loads(
            processing_resource.response_data
        )
        return make_response(jsonify(response_model), html_code)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("delete", resource_management.resource_delete_doc)
    )
    def delete(self, user_id, resource_id):
        """Request the termination of a resource."""

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        if not resource_id.startswith("resource_id-"):
            resource_id = "resource_id-%s" % resource_id

        iteration, doc = self.resource_logger.get_latest_iteration(
            user_id, resource_id
        )

        if doc is None:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource does not exist"
                    )
                ),
                400,
            )

        self.resource_logger.commit_termination(
            user_id, resource_id, iteration
        )

        return make_response(
            jsonify(
                SimpleResponseModel(
                    status="accepted", message="Termination request committed"
                )
            ),
            200,
        )


# Create a g.list/g.remove pattern parser
resource_parser = reqparse.RequestParser()
resource_parser.add_argument(
    "num",
    type=int,
    help="The maximum number of jobs that should be listed",
    location="args",
)
resource_parser.add_argument(
    "type",
    type=str,
    help="The type of the jobs that should be shown: "
    "all, running, error, terminated, finished",
    location="args",
)


class ResourcesManager(ResourceManagerBase):
    """Management of multiple resources

    TODO: This methods must be secured by checking the user id. Only admins
    can terminate and list resources from other users.

    """

    def __init__(self):
        # Configuration
        ResourceManagerBase.__init__(self)

    def _get_resource_list(self, user_id, type_="all"):
        """
        Get a list of resources that have been generated by the calling user
        """
        parsed_list = []
        resource_list = self.resource_logger.get_user_resources(user_id)

        for entry in resource_list:
            if type_.lower() == "all":
                parsed_list.append(entry)
            elif type_.lower() == entry["status"]:
                parsed_list.append(entry)

        return parsed_list

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", resource_management.resources_get_doc))
    def get(self, user_id):
        """
        Get a list of resources that have been generated by the specified user.
        """

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        args = resource_parser.parse_args()
        num = None
        if "num" in args and args["num"]:
            num = args["num"]
        type_ = "all"
        if "type" in args and args["type"]:
            type_ = args["type"]

        resource_list = self._get_resource_list(user_id, type_=type_)

        if num is not None:
            response_list = resource_list[0:num]
        else:
            response_list = resource_list

        return make_response(
            jsonify(ProcessingResponseListModel(resource_list=response_list)),
            200,
        )

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("delete", resource_management.resources_delete_doc)
    )
    def delete(self, user_id):
        """
        Terminate all accepted and running resources of the specified user.
        """

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        resource_list = self._get_resource_list(user_id)

        termination_requests = 0
        for entry in resource_list:
            if "status" in entry:
                if entry["status"] in ["accepted", "running"]:
                    self.resource_logger.commit_termination(
                        user_id, entry["resource_id"]
                    )
                    termination_requests += 1

        return make_response(
            jsonify(
                SimpleResponseModel(
                    status="finished",
                    message="Successfully send %i termination requests"
                    % termination_requests,
                )
            ),
            200,
        )


class ResourceIterationManager(ResourceManagerBase):
    """
    This class is responsible to answer status requests
    of asynchronous processes (resources) and
    to request the termination of a resource with iterations
    """

    def __init__(self):
        # Configuration
        ResourceManagerBase.__init__(self)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("get", resource_management.resource_iteration_get_doc)
    )
    def get(self, user_id, resource_id, iteration):
        """Get the status of a resource of a given iteration."""
        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        if not resource_id.startswith("resource_id-"):
            resource_id = "resource_id-%s" % resource_id

        response_data = self.resource_logger.get(
            user_id, resource_id, int(iteration)
        )

        if response_data is not None:
            _, tmp_response_model = pickle.loads(response_data)
            response_model = {str(iteration): tmp_response_model}
            return make_response(jsonify(response_model), 200)
        else:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource does not exist"
                    )
                ),
                400,
            )



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
User management base
"""

from actinia_core.core.common.config import global_config

__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis GmbH & Co. KG"


USER_ROLES = ["superadmin", "admin", "user", "guest"]


class ActiniaUserError(Exception):
    """Raise this exception in case a user creation error happens"""

    def __init__(self, message):
        message = "%s:  %s" % (str(self.__class__.__name__), message)
        Exception.__init__(self, message)


class ActiniaUserBase(object):
    """
    The Actinia Core user management base class

    This class manages the user base where the users stored in Redis database
    or keycloak are based on.
    """

    def __init__(
        self,
        user_id,
        user_group=None,
        user_role=None,
        accessible_datasets={
            "nc_spm_08": ["PERMANENT", "user1", "landsat"],
            "ECAD": ["PERMANENT"],
            "latlong_wgs84": ["PERMANENT"],
        },
        accessible_modules=global_config.MODULE_ALLOW_LIST,
        cell_limit=global_config.MAX_CELL_LIMIT,
        process_num_limit=global_config.PROCESS_NUM_LIMIT,
        process_time_limit=global_config.PROCESS_TIME_LIMT,
    ):
        """Constructor

        Initialize and create a user object.

        Args:
            user_id (str): The id (name, email, ..) of the user that must be
                           unique
            user_group (str): The group of the user
            user_role (str): The user role (superadmin, admin, user, guest)
            accessible_datasets (dict): Dict of location:mapset lists
            accessible_modules (list): A list of modules that are allowed to
                                       use
            cell_limit (int): Maximum number of cells to process
            process_num_limit (int): The maximum number of processes the user
                                     is allowed to run in a single chain
            process_time_limit (int): The maximum number of seconds a user
                                      process is allowed to run

        """

        self.user_id = user_id
        self.user_group = user_group
        self.password_hash = None
        self.user_role = None
        self.permissions = None
        self.cell_limit = None
        self.accessible_datasets = {}
        self.accessible_modules = []
        self.process_num_limit = None
        self.process_time_limit = None

        if user_role:
            self.set_role(user_role)
        if accessible_datasets is not None:
            self.set_accessible_datasets(accessible_datasets)
        if accessible_modules is not None:
            self.set_accessible_modules(accessible_modules)
        if cell_limit is not None:
            self.set_cell_limit(cell_limit)
        if process_num_limit is not None:
            self.set_process_num_limit(process_num_limit)
        if process_time_limit is not None:
            self.set_process_time_limit(process_time_limit)

    def _generate_permission_dict(self):
        """Create the permission dictionary"""

        self.permissions = {
            "accessible_datasets": self.accessible_datasets,
            "accessible_modules": self.accessible_modules,
            "cell_limit": self.cell_limit,
            "process_num_limit": self.process_num_limit,
            "process_time_limit": self.process_time_limit,
        }

    def set_role(self, role):
        """Set the user role

        Args:
            role: The user role, can be admin, user or guest

        Raises:
            ActiniaUserError in case the role is not supported

        The following roles are supported:
        USER_ROLES = ["superadmin",
                      "admin",
                      "user",
                      "guest"]
        """
        if role not in USER_ROLES:
            raise ActiniaUserError(
                "Unsupported user role <%s> supported are %s"
                % (role, str(USER_ROLES))
            )
        self.user_role = role

    def get_role(self):
        return self.user_role

    def has_guest_role(self):
        return self.get_role() == "guest"

    def has_user_role(self):
        return self.get_role() == "user"

    def has_admin_role(self):
        return self.get_role() == "admin"

    def has_superadmin_role(self):
        return self.get_role() == "superadmin"

    def set_group(self, group):
        """Set the user group

        Args:
            group: The user group

        """
        self.user_group = group

    def set_accessible_datasets(self, accessible_datasets):
        """Set the accessible datasets

        Args:
            accessible_datasets (dict):

        Example::

            {"nc_spm_08":["PERMANENT",
                          "user1",
                          "landsat"],
             "utm32N":["PERMANENT",
                       "sentinel2_bonn"]
            }

        """
        self.accessible_datasets = accessible_datasets

    def add_accessible_dataset(self, location_name, mapset_list):
        """Add a dataset to the accessible datasets

        If the dataset exists, the mapsets will be extended by the provided
        list

        Args:
            location_name (str): Location name
            mapset_list (list): List of mapset names

        Example::

            location_name="nc_spm_08"

            mapset_list = ["PERMANENT",
                          "user1",
                          "landsat"]

        """

        if location_name not in self.accessible_datasets:
            self.accessible_datasets[location_name] = mapset_list
        else:
            for mapset in mapset_list:
                if mapset not in self.accessible_datasets[location_name]:
                    self.accessible_datasets[location_name].append(mapset)

    def remove_mapsets_from_location(self, location_name, mapset_list):
        """Remove mapsets from an existing location

        Args:
            location_name (str): Location name
            mapset_list (list): List of mapset names that should be removed

        Example::

            location_name="nc_spm_08"

            mapset_list = ["landsat",]

        """

        if location_name in self.accessible_datasets:
            for mapset in mapset_list:
                if mapset in self.accessible_datasets[location_name]:
                    self.accessible_datasets[location_name].remove(mapset)

    def remove_location(self, location_name):
        """Remove a location from the accessible datasets

        Args:
            location_name (str): Location name

        Example::

            location_name="nc_spm_08"

        """

        if location_name in self.accessible_datasets:
            self.accessible_datasets.pop(location_name)

    def set_accessible_modules(self, accessible_modules):
        """Set the accessible modules

        Args:
            accessible_modules (list): A list of module names

        Example::

            ["g.region", "i.vi"]

        """
        self.accessible_modules = accessible_modules

    def add_accessible_modules(self, module_names):
        """Set the accessible modules

        Args:
            module_names (list): A list of module names

        Example::

            ["g.region", "i.vi"]

        """
        for name in module_names:
            if name not in self.accessible_modules:
                self.accessible_modules.append(name)

    def remove_accessible_modules(self, module_names):
        """Remove accessible modules from the list

        Args:
            module_names (list): A list of module names that should be removed

        Example::

            ["g.region", "i.vi"]

        """
        for name in module_names:
            if name in self.accessible_modules:
                self.accessible_modules.remove(name)

    def set_cell_limit(self, cell_limit):
        """Set the maximum number of cells that the user is allowed to process

        Args:
            cell_limit (int):

        Raises:
            ActiniaUserError in case the cell_limit is not supported
        """

        try:
            self.cell_limit = int(cell_limit)
        except Exception:
            raise ActiniaUserError("Wrong format for cell limit")

    def set_process_num_limit(self, process_num_limit):
        """Set the maximum number of processes that the user is allowed to run
        in a single process chain

        Args:
            process_num_limit (int):

        Raises:
            ActiniaUserError in case the cell_limit is not supported
        """

        try:
            self.process_num_limit = int(process_num_limit)
        except Exception:
            raise ActiniaUserError("Wrong format for process_num_limit")

    def set_process_time_limit(self, process_time_limit):
        """Set the maximum number of seconds that the user is allowed to run
        a single process

        Args:
            process_time_limit (int):

        Raises:
            ActiniaUserError in case the process_time_limit is not supported
        """

        try:
            self.process_time_limit = int(process_time_limit)
        except Exception:
            raise ActiniaUserError("Wrong format for process_time_limit")

    def get_id(self):
        return self.user_id



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
STRDS raster map layer management

TODO: Integrate into the ephemeral process chain approach
"""
from flask_restful_swagger_2 import swagger
from flask import jsonify, make_response
import pickle
from actinia_api.swagger2.actinia_core.apidocs import strds_raster_management

from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.core.request_parser import where_parser
from actinia_core.rest.base.resource_base import ResourceBase
from actinia_core.processing.common.strds_raster_management import (
    unregister_raster,
    register_raster,
    list_raster_strds,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class STRDSRasterManagement(ResourceBase):
    """Manage raster layer in a space time raster dataset"""

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", strds_raster_management.get_doc))
    def get(self, location_name, mapset_name, strds_name):
        """
        Get a list of all raster map layers that are registered in a STRDS
        """
        rdc = self.preprocess(
            has_json=False,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=strds_name,
        )

        if rdc:
            args = where_parser.parse_args()
            rdc.set_user_data(args)

            enqueue_job(
                self.job_timeout,
                list_raster_strds,
                rdc,
                queue_type_overwrite=True,
            )
            http_code, response_model = self.wait_until_finish()
        else:
            http_code, response_model = pickle.loads(self.response_data)

        return make_response(jsonify(response_model), http_code)

    @endpoint_decorator()
    @swagger.doc(check_endpoint("put", strds_raster_management.put_doc))
    def put(self, location_name, mapset_name, strds_name):
        """Register raster map layers in a STRDS located in a specific
        location/mapset.
        """
        rdc = self.preprocess(
            has_json=True,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=strds_name,
        )

        if rdc:
            enqueue_job(self.job_timeout, register_raster, rdc)
            http_code, response_model = self.wait_until_finish()
        else:
            http_code, response_model = pickle.loads(self.response_data)

        return make_response(jsonify(response_model), http_code)

    @endpoint_decorator()
    @swagger.doc(check_endpoint("delete", strds_raster_management.delete_doc))
    def delete(self, location_name, mapset_name, strds_name):
        """
        Unregister raster map layers from a STRDS located in a specific
        location/mapset.
        """
        rdc = self.preprocess(
            has_json=True,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=strds_name,
        )

        if rdc:
            enqueue_job(
                self.job_timeout,
                unregister_raster,
                rdc,
                queue_type_overwrite=True,
            )
            http_code, response_model = self.wait_until_finish()
        else:
            http_code, response_model = pickle.loads(self.response_data)

        return make_response(jsonify(response_model), http_code)
